/**
 * Maling.java  - "Programmering i Java", 4.utgave - 2009-07-01
 *
 * Klassen Maling er en spesialisering av Materiale.
 */

class Maling extends Materiale {
  private static final double GRENSE = 0.02;  // grense for  kjpe 0.5 liter maling til
  private final int antStrk;
  private final double antKvmPrLiter;

  public Maling(String navn, double pris, int antStrk, double antKvmPrLiter) {
    super(navn, pris);
    if (antStrk <= 0 || antKvmPrLiter <= 0.0) {
      throw new IllegalArgumentException(
                "Bde ant. strk og dekkevne (kvm/l) m vre positive tall.\n"
             + "Inndata til konstruktren var: ant. strk = "
             + antStrk + ", dekkevne (kvm/l) = " + antKvmPrLiter);
    }
    this.antStrk = antStrk;
    this.antKvmPrLiter = antKvmPrLiter;
  }

  public int getAntStrk() {
    return antStrk;
  }

  public double getAntKvmPrLiter() {
    return antKvmPrLiter;
  }

  /**
   * Metoden finner antall liter maling som trengs,
   * behovet rundes oppover til nrmeste halve liter.
   * Metoden returnerer et negativt tall, dersom enFlate er null.
   */
  public double beregnMaterialbehov(Flate enFlate) {
    if (enFlate != null) {
      double areal = enFlate.beregnAreal();
      double antLiter = areal * antStrk / antKvmPrLiter;
      int heltAntallLitere = (int) antLiter;
      double overLiteren = antLiter - heltAntallLitere;
      if (overLiteren >= 0.5 + GRENSE) {
        return heltAntallLitere + 1.0;
      } else {
        if (overLiteren >= GRENSE) {
          return heltAntallLitere + 0.5;
        } else {
          return heltAntallLitere;
        }
      }
    } else {
      return -1.0;
    }
  }

  public String toString() {
    java.util.Formatter f = new java.util.Formatter();
    f.format("Maling: %s pr. liter, ant. strk: %d, ant. kvm/l %.2f",
                 super.toString(), antStrk, antKvmPrLiter);
    return f.toString();
  }
}